@@ -20,6 +20,8 @@
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
+#include <ctype.h>
+#include <math.h>
 
 #include "miniexp.h"
 
@@ -32,9 +34,9 @@ miniexp_t s_true = miniexp_symbol("t");
 /* ------------ error */
 
 #ifdef __GNUC__
-void
-error(const char *msg, miniexp_t v=0)
-  __attribute__ ((noreturn));
+void error(const char *msg, miniexp_t v=0) __attribute__ ((noreturn));
+#else
+void error(const char *msg, miniexp_t v=0);
 #endif
 
 void
@@ -553,6 +555,10 @@ DEFUN("numberp",numberp,1,0) {
   return miniexp_numberp(argv[0]) ? s_true : 0;
 }
 
+DEFUN("doublep",doublep,1,0) {
+  return miniexp_doublep(argv[0]) ? s_true : 0;
+}
+
 DEFUN("objectp",objectp,1,0) {
   return miniexp_objectp(argv[0]) ? s_true : 0;
 }
@@ -607,7 +613,7 @@ DEFUN("cons",cons,2,0) {
 
 DEFUN("nth",nth,2,0) {
   if (! miniexp_numberp(argv[0]))
-    error("nth: number expected");
+    error("nth: integer number expected");
   return miniexp_nth(miniexp_to_int(argv[0]), argv[1]);
 }
 
@@ -620,60 +626,60 @@ DEFUN("rplacd",rplacd,2,0) {
 }
 
 DEFUN("abs",abs,1,0) {
-  return miniexp_number(abs(miniexp_to_int(argv[0])));
+  return miniexp_double(fabs(miniexp_to_double(argv[0])));
 }
 
 DEFUN("+",plus,0,9999) {
-  int s = 0;
+  double s = 0;
   for (int i=0; i<argc; i++)
     {
-      if (!miniexp_numberp(argv[i]))
+      if (!miniexp_doublep(argv[i]))
 	error("+: number expected");
-      s += miniexp_to_int(argv[i]);
+      s += miniexp_to_double(argv[i]);
     }
-  return miniexp_number(s);
+  return miniexp_double(s);
 }
 
 DEFUN("*",times,0,9999) {
-  int s = 1;
+  double s = 1;
   for (int i=0; i<argc; i++)
     {
-      if (!miniexp_numberp(argv[i]))
+      if (!miniexp_doublep(argv[i]))
 	error("*: number expected");
-      s *= miniexp_to_int(argv[i]);
+      s *= miniexp_to_double(argv[i]);
     }
-  return miniexp_number(s);
+  return miniexp_double(s);
 }
 
 DEFUN("-",minus,1,9999) {
-  if (! miniexp_numberp(argv[0]))
+  if (! miniexp_doublep(argv[0]))
     error("-: number expected");
   int i = 0;
-  int s = 0;
-  if (argc>1 && miniexp_numberp(argv[0]))
-    s = miniexp_to_int(argv[i++]);
-  while (i<argc && miniexp_numberp(argv[i]))
-    s -= miniexp_to_int(argv[i++]);
+  double s = 0;
+  if (argc>1 && miniexp_doublep(argv[0]))
+    s = miniexp_to_double(argv[i++]);
+  while (i<argc && miniexp_doublep(argv[i]))
+    s -= miniexp_to_double(argv[i++]);
   if (i < argc)
     error("-: number expected", argv[i]);
-  return miniexp_number(s);
+  return miniexp_double(s);
 }
 
 DEFUN("/",div,1,9999) {
-  if (! miniexp_numberp(argv[0]))
+  if (! miniexp_doublep(argv[0]))
     error("/: number expected");
   int i = 0;
-  int s = 1;
-  if (argc>1 && miniexp_numberp(argv[0]))
-    s = miniexp_to_int(argv[i++]);
-  while (i<argc && miniexp_numberp(argv[i]) && miniexp_to_int(argv[i]))
-    s /= miniexp_to_int(argv[i++]);
+  double s = 1;
+  if (argc>1 && miniexp_doublep(argv[0]))
+    s = miniexp_to_double(argv[i++]);
+  while (i<argc && miniexp_doublep(argv[i]) && miniexp_to_double(argv[i]))
+    s /= miniexp_to_double(argv[i++]);
   if (i < argc)
-    if (miniexp_numberp(argv[i]))
+    if (miniexp_doublep(argv[i]))
       error("/: division by zero", argv[i]);
     else
       error("/: number expected", argv[i]);
-  return miniexp_number(s);
+  return miniexp_double(s);
 }
 
 DEFUN("==",equalequal,2,0) {
@@ -690,6 +696,8 @@ equal(miniexp_t a, miniexp_t b)
       &&   equal(miniexp_cdr(a),miniexp_cdr(b));
   else if (miniexp_stringp(a) && miniexp_stringp(b))
     return !strcmp(miniexp_to_str(a), miniexp_to_str(b));
+  else if (miniexp_doublep(a) && miniexp_doublep(b))
+    return miniexp_to_double(a) == miniexp_to_double(b);
   return false;
 }
 
@@ -704,10 +712,10 @@ DEFUN("<>",notequal,2,0) {
 static int
 compare(miniexp_t a, miniexp_t b)
 {
-  if (miniexp_numberp(a) && miniexp_numberp(b))
+  if (miniexp_doublep(a) && miniexp_doublep(b))
     {
-      int na = miniexp_to_int(a);
-      int nb = miniexp_to_int(b);
+      double na = miniexp_to_double(a);
+      double nb = miniexp_to_double(b);
       if (na < nb)
 	return -1;
       else if (na > nb)
@@ -740,22 +748,41 @@ DEFUN(">",cmpgt,2,0) {
   return (compare(argv[0],argv[1])>0) ? s_true : 0;
 }
 
+DEFUN("floor",floor,1,0) {
+  if (! miniexp_doublep(argv[0]))
+    error("-: number expected");
+  return miniexp_double(floor(miniexp_to_double(argv[0])));
+}
+
+DEFUN("ceil",ceil,1,0) {
+  if (! miniexp_doublep(argv[0]))
+    error("-: number expected");
+  return miniexp_double(ceil(miniexp_to_double(argv[0])));
+}
+
+DEFUN("strlen",strlen,1,1) {
+  if (! miniexp_stringp(argv[0]))
+    error("strlen: string expected", argv[0]);
+  const char *s = miniexp_to_str(argv[0]);
+  return miniexp_number(strlen(s));
+}
+
 DEFUN("substr",substr,2,1) {
   if (! miniexp_stringp(argv[0]))
     error("substr: string expected", argv[0]);
   const char *s = miniexp_to_str(argv[0]);
   int l = strlen(s);
   if (! miniexp_numberp(argv[1]))
-    error("substr: number expected", argv[1]);
-  int f = miniexp_to_int(argv[1]);
+    error("substr: integer number expected", argv[1]);
+  int f = miniexp_to_double(argv[1]);
   f = (l < f) ? l : (f < 0) ? l : f;
   s += f;
   l -= f;
   if (argc>2)
     {
       if (! miniexp_numberp(argv[2]))
-	error("substr: number expected", argv[2]);
-      f = miniexp_to_int(argv[2]);
+	error("substr: integer number expected", argv[2]);
+      f = miniexp_to_double(argv[2]);
       l = (f > l) ? l : (f < 0) ? 0 : f;
     }
   return miniexp_substring(s,l);
@@ -884,12 +911,117 @@ DEFUN("symbol->string",symbol2string,1,0) {
   return miniexp_string(miniexp_to_name(argv[0]));
 }
 
+DEFUN("printflags",printflags,1,0) {
+  if (! miniexp_numberp(argv[0]))
+    error("printflags: integer number expected");
+  minilisp_print_7bits = miniexp_to_int(argv[0]);
+  return argv[0];
+}
+
+/* ------------ special */
+
+#if defined(_WIN32) || defined(__WIN64)
+# include <process.h>
+
+class thread_t : public miniobj_t
+{
+  MINIOBJ_DECLARE(thread_t, miniobj_t, "thread");
+private:
+  uintptr_t thr;
+  miniexp_t exp, env, res, run;
+  static void start(void *arg) {
+    thread_t *pth = (thread_t*) arg;
+    try { 
+      pth->res = evaluate(pth->exp, pth->env); 
+      pth->run = miniexp_symbol("finished");
+    } catch(...) { 
+      pth->run = miniexp_symbol("error");
+    } }
+public:
+  thread_t(miniexp_t exp, miniexp_t env) : exp(exp), env(env), res(0), run(0) { 
+    thr = _beginthread(thread_t::start, 0, (void*)this); }
+  void mark(minilisp_mark_t action) {
+    action(&exp); action(&env), action(&res); }
+  miniexp_t join() {
+    return (run) ? res : miniexp_dummy; }
+  miniexp_t status() { return run; }
+  ~thread_t() { if (!run) abort(); join(); }
+};
+
+MINIOBJ_IMPLEMENT(thread_t, miniobj_t, "thread");
+
+DEFUN("thread",threadstart,1,0) {
+  return miniexp_object(new thread_t(argv[0],env));
+}
+DEFUN("threadp", threadtest,1,0) {
+  if (! miniexp_isa(argv[0], thread_t::classname)) return 0;
+  miniexp_t run = ((thread_t*)miniexp_to_obj(argv[0]))->status();
+  return (run) ? run : miniexp_symbol("running");
+}
+DEFUN("join",threadjoin,1,0) {
+  if (! miniexp_isa(argv[0], thread_t::classname))
+    error("join: thread expected");
+  return ((thread_t*)miniexp_to_obj(argv[0]))->join();
+}
+#endif
+
+#ifdef HAVE_PTHREAD
+# include <pthread.h>
+
+class thread_t : public miniobj_t
+{
+  MINIOBJ_DECLARE(thread_t, miniobj_t, "thread");
+private:
+  pthread_t thr;
+  miniexp_t exp, env, res, run;
+  bool joined;
+  static void* start(void *arg) {
+    thread_t *pth = (thread_t*) arg;
+    try { 
+      pth->res = evaluate(pth->exp, pth->env); 
+      pth->run = miniexp_symbol("finished");
+      return 0; } 
+    catch(...) { 
+      pth->run = miniexp_symbol("error");
+      return (void*)1; } }
+public:
+  thread_t(miniexp_t exp, miniexp_t env) 
+    : exp(exp), env(env), res(0), run(0), joined(false) { 
+    pthread_create(&this->thr, 0, thread_t::start, (void*)this); }
+  void mark(minilisp_mark_t action) {
+    action(&exp); action(&env), action(&res); }
+  miniexp_t join() {
+    if (! joined) pthread_join(thr, 0); joined=true;
+    return (run) ? res : miniexp_dummy; }
+  miniexp_t status() { return run; }
+  ~thread_t() { if (!run) abort(); join(); }
+};
+
+MINIOBJ_IMPLEMENT(thread_t, miniobj_t, "thread");
+
+DEFUN("thread",threadstart,1,0) {
+  return miniexp_object(new thread_t(argv[0],env));
+}
+DEFUN("threadp", threadtest,1,0) {
+  if (! miniexp_isa(argv[0], thread_t::classname)) return 0;
+  miniexp_t run = ((thread_t*)miniexp_to_obj(argv[0]))->status();
+  return (run) ? run : miniexp_symbol("running");
+}
+DEFUN("join",threadjoin,1,0) {
+  if (! miniexp_isa(argv[0], thread_t::classname))
+    error("join: thread expected");
+  return ((thread_t*)miniexp_to_obj(argv[0]))->join();
+}
+
+#endif
+
 
 /* ------------ toplevel */
 
 void
 toplevel(FILE *inp, FILE *out, bool print)
 {
+  miniexp_io_t saved_io = miniexp_io;
   minilisp_set_output(out);
   minilisp_set_input(inp);
   for(;;)
@@ -897,10 +1029,9 @@ toplevel(FILE *inp, FILE *out, bool print)
       minivar_t s = miniexp_read();
       if (s == miniexp_dummy)
 	{
-	  if (feof(inp))
-	    break;
-	  printf("ERROR: while parsing\n");
-	  continue;
+          if (feof(inp)) break;
+          printf("ERROR: while parsing\n");
+	  break;
 	}
       try
 	{
@@ -916,6 +1047,7 @@ toplevel(FILE *inp, FILE *out, bool print)
 	{
 	}
     }
+  miniexp_io = saved_io;
 }
 
 miniexp_t
@@ -931,6 +1063,8 @@ miniexp_t
 parse_quote(void)
 {
   minivar_t l = miniexp_read();
+  if (l == miniexp_dummy)
+    return miniexp_dummy;
   l = miniexp_cons(s_quote, miniexp_cons(l, miniexp_nil));
   return miniexp_cons(l,miniexp_nil);
 }
@@ -942,6 +1076,20 @@ sighandler(int signo)
   signal(signo, sighandler);
 }
 
+DEFUN("load",xload,1,0) {
+  if (! miniexp_stringp(argv[0]))
+    error("load: string expected");
+  FILE *f = fopen(miniexp_to_str(argv[0]), "r");
+  if (! f)
+    error("load: cannot open file");
+  toplevel(f, stdout, false);
+  fclose(f);
+  return miniexp_nil;
+}
+
+
+/* ------------ toplevel */
+
 int
 main()
 {
@@ -951,15 +1099,15 @@ main()
   minilisp_macrochar_parser[(int)';'] = parse_comment;
   minilisp_macrochar_parser[(int)'\''] = parse_quote;
   FILE *f = fopen("minilisp.in","r");
-  if (f)
-    {
-      toplevel(f, stdout, false);
-      fclose(f);
-    }
-  else
+  if (f) {
+    toplevel(f, stdout, false);
+    fclose(f);
+  } else
     printf("WARNING: cannot find 'minilisp.in'\n");
   signal(SIGINT, sighandler);
-  toplevel(stdin, stdout, true);
+  while (! feof(stdin))
+    toplevel(stdin, stdout, true);
+  break_request = true;
   minilisp_finish();
   return 0;
 }
