@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: ByteStream.h,v 1.16 2007/05/19 03:07:33 leonb Exp $
-// $Name: release_3_5_23 $
 
 #ifndef _BYTESTREAM_H
 #define _BYTESTREAM_H
@@ -97,9 +94,7 @@
 // Our original implementation consisted of multiple classes.
 // <http://prdownloads.sourceforge.net/djvu/DjVu2_2b-src.tgz>.
 
-
-    @version
-    #$Id: ByteStream.h,v 1.16 2007/05/19 03:07:33 leonb Exp $# */
+*/
 //@{
 
 
@@ -272,13 +267,13 @@ public:
   /** Returns the total number of bytes contained in the buffer, file, etc.
       Valid offsets for function #seek# range from 0 to the value returned
       by this function. */
-  virtual int size(void) const;
+  virtual long size(void) const;
   /// Use at your own risk, only guarenteed to work for ByteStream::Memorys.
   TArray<char> get_data(void);
   /** Reads data from a random position. This function reads at most #sz#
       bytes at position #pos# into #buffer# and returns the actual number of
       bytes read.  The current position is unchanged. */
-  virtual size_t readat(void *buffer, size_t sz, int pos);
+  virtual size_t readat(void *buffer, size_t sz, long pos);
   //@}
 protected:
   ByteStream(void) : cp(AUTO) {};
@@ -346,7 +341,7 @@ public:
 };
 
 inline size_t
-ByteStream::readat(void *buffer, size_t sz, int pos)
+ByteStream::readat(void *buffer, size_t sz, long pos)
 {
   size_t retval;
   long tpos=tell();
@@ -356,17 +351,17 @@ ByteStream::readat(void *buffer, size_t sz, int pos)
   return retval;
 }
 
-inline int
+inline long
 ByteStream::size(void) const
 {
   ByteStream *bs=const_cast<ByteStream *>(this);
-  int bsize=(-1);
-  long pos=tell();
+  long bsize = -1;
+  long pos = tell();
   if(bs->seek(0,SEEK_END,true))
-  {
-    bsize=(int)tell();
-    (void)(bs->seek(pos,SEEK_SET,false));
-  }
+    {
+      bsize = tell();
+      (void)(bs->seek(pos,SEEK_SET,false));
+    }
   return bsize;
 }
 
