@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: DjVuDocument.cpp,v 1.21 2008/08/05 20:50:35 bpearlmutter Exp $
-// $Name: release_3_5_22 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -179,6 +176,7 @@ DjVuDocument::start_init(
       // Initialize
    cache=xcache;
    doc_type=UNKNOWN_TYPE;
+   DataPool::close_all();
    DjVuPortcaster * pcaster=get_portcaster();
    if (!xport)
      xport=simple_port=new DjVuSimplePort();
@@ -1654,7 +1652,7 @@ DjVuDocument::get_url_names(void)
         // Why is this try/catch block here?
         G_TRY { 
           get_portcaster()->notify_error(this, ex.get_cause()); 
-          GUTF8String emsg = ERR_MSG("DjVuDocument.exclude_page") "\t" + (i+1);
+          GUTF8String emsg = ERR_MSG("DjVuDocument.exclude_page") "\t" + GUTF8String(i+1);
           get_portcaster()->notify_error(this, emsg);
         }
         G_CATCH_ALL
@@ -1743,7 +1741,7 @@ DjVuDocument::get_djvm_doc()
                    G_TRY { 
                      get_portcaster()->notify_error(this, ex.get_cause());
                      GUTF8String emsg = ERR_MSG("DjVuDocument.skip_page") "\t" 
-                                      + (page_num+1);
+                                      + GUTF8String(page_num+1);
                      get_portcaster()->notify_error(this, emsg);
                    }
                    G_CATCH_ALL
@@ -1776,10 +1774,20 @@ DjVuDocument::write(const GP<ByteStream> &gstr, bool force_djvm)
    
   GP<DjVmDoc> doc=get_djvm_doc();
   GP<DjVmDir> dir=doc->get_djvm_dir();
-  if (force_djvm || dir->get_files_num()>1)
+
+  bool singlepage = (dir->get_files_num()==1 && !djvm_nav && !force_djvm);
+  if (singlepage)
+  {
+    // maybe save as single page
+    DjVmDir::File *file = dir->page_to_file(0);
+    if (file->get_title() != file->get_load_name())
+      singlepage = false;
+  }
+  if (! singlepage)
   {
     doc->write(gstr);
-  }else
+  }
+  else
   {
     GPList<DjVmDir::File> files_list=dir->resolve_duplicates(false);
     GP<DataPool> pool=doc->get_data(files_list[files_list]->get_load_name());
@@ -1829,18 +1837,21 @@ DjVuDocument::save_as(const GURL &where, bool bundled)
    }
 }
 
-static const char prolog[]="<?xml version=\"1.0\" ?>\n<!DOCTYPE DjVuXML PUBLIC \"-//W3C//DTD DjVuXML 1.1//EN\" \"pubtext/DjVuXML-s.dtd\">\n<DjVuXML>\n<HEAD>";
+static const char prolog[]="<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE DjVuXML PUBLIC \"-//W3C//DTD DjVuXML 1.1//EN\" \"pubtext/DjVuXML-s.dtd\">\n<DjVuXML>\n<HEAD>";
 static const char start_xml[]="</HEAD>\n<BODY>\n";
 static const char end_xml[]="</BODY>\n</DjVuXML>\n";
 
 void
-DjVuDocument::writeDjVuXML(const GP<ByteStream> &gstr_out,int flags) const
+DjVuDocument::writeDjVuXML(const GP<ByteStream> &gstr_out,
+                           int flags, int page) const
 {
   ByteStream &str_out=*gstr_out;
   str_out.writestring(
     prolog+get_init_url().get_string().toEscaped()+start_xml);
   const int pages=wait_get_pages_num();
-  for(int page_num=0;page_num<pages;++page_num)
+  int pstart = (page < 0) ? 0 : page;
+  int pend = (page < 0) ? pages : page+1;
+  for(int page_num=pstart; page_num<pend; ++page_num)
   {
     const GP<DjVuImage> dimg(get_page(page_num,true));
     if(!dimg)
