@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: GContainer.cpp,v 1.15 2007/03/25 20:48:31 leonb Exp $
-// $Name: release_3_5_23 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -740,15 +737,10 @@ GSetBase::rehash(int newbuckets)
   nelems = 0;
   first = 0;
   // Allocate a new empty bucket table
-// delete [] table;
   gtable.resize(0);
   nbuckets = newbuckets;
-  typedef HNode *HNodePtr;
-// table = new HNodePtr[nbuckets];
   gtable.resize(nbuckets);
   gtable.clear();
-//  for (int i=0; i<nbuckets; i++)
-//    table[i] = 0;
   // Insert saved nodes
   while (n)
     {
