@@ -76,7 +76,7 @@
 # define UNIX 1
 #endif
 
-#if defined(WIN32) && !defined(UNIX)
+#if defined(_WIN32) && !defined(UNIX)
 # include <windows.h>
 # include <string.h>
 # include <direct.h>
@@ -169,7 +169,7 @@ static const char nillchar=0;
 static inline int
 finddirsep(const GUTF8String &fname)
 {
-#if defined(WIN32)
+#if defined(_WIN32)
   return fname.rcontains("\\/",0);
 #elif defined(UNIX)
   return fname.rsearch('/',0);
@@ -191,7 +191,7 @@ GOS::basename(const GUTF8String &gfname, const char *suffix)
     return gfname;
 
   const char *fname=gfname;
-#if defined(WIN32) || defined(OS2)
+#if defined(_WIN32) || defined(OS2)
   // Special cases
   if (fname[1] == colon)
   {
@@ -272,7 +272,7 @@ GOS::ticks()
     G_THROW(errmsg());
   return (unsigned long)( ((tv.tv_sec & 0xfffff)*1000) 
                           + (tv.tv_usec/1000) );
-#elif defined(WIN32)
+#elif defined(_WIN32)
   DWORD clk = GetTickCount();
   return (unsigned long)clk;
 #elif defined(OS2)
@@ -295,12 +295,8 @@ GOS::sleep(int milliseconds)
   struct timeval tv;
   tv.tv_sec = milliseconds / 1000;
   tv.tv_usec = (milliseconds - (tv.tv_sec * 1000)) * 1000;
-# if defined(THREADMODEL) && (THREADMODEL==COTHREADS)
-  GThread::select(0, NULL, NULL, NULL, &tv);
-# else
-  select(0, NULL, NULL, NULL, &tv);
-# endif
-#elif defined(WIN32)
+  ::select(0, NULL, NULL, NULL, &tv);
+#elif defined(_WIN32)
   Sleep(milliseconds);
 #elif defined(OS2)
   DosSleep(milliseconds);
@@ -335,7 +331,7 @@ GOS::cwd(const GUTF8String &dirname)
   if (!result)
     G_THROW(errmsg());
   return GNativeString(result).getNative2UTF8();//MBCS cvt
-#elif defined (WIN32)
+#elif defined(_WIN32)
   char drv[2];
   if (dirname.length() && _chdir(dirname.getUTF82Native())==-1)//MBCS cvt
     G_THROW(errmsg());
