@@ -79,7 +79,7 @@
 #include <math.h>
 #include <string.h>
 
-#ifdef WIN32
+#ifdef _WIN32
 # include <tchar.h>
 # include <windows.h>
 # include <direct.h>
@@ -183,7 +183,7 @@ static const char nillchar=0;
 #if defined(UNIX)
   static const char tilde='~';
   static const char root[] = "/";
-#elif defined(WIN32) || defined(OS2)
+#elif defined(_WIN32) || defined(OS2)
   static const char root[] = "\\";
 #elif defined(macintosh)
   static char const * const root = &nillchar; 
@@ -232,7 +232,7 @@ void
 GURL::convert_slashes(void)
 {
    GUTF8String xurl(get_string());
-#if defined(WIN32)
+#if defined(_WIN32)
    const int protocol_length=protocol(xurl).length();
    for(char *ptr=(xurl.getbuf()+protocol_length);*ptr;ptr++)
      if(*ptr == backslash)
@@ -288,7 +288,7 @@ GURL::beautify_path(GUTF8String xurl)
   // Convert /./ stuff into plain /
   for(;(ptr=strstr(start, "/./"));collapse(ptr, 2))
     EMPTY_LOOP;
-#if defined(WIN32) || defined(OS2)
+#if defined(_WIN32) || defined(OS2)
   if(!xurl.cmp(filespec,sizeof(filespec)-1))
   {
 	int offset=1;
@@ -300,7 +300,7 @@ GURL::beautify_path(GUTF8String xurl)
 	}
     for(ptr=start+offset;(ptr=strchr(ptr, '/'));)
 	{
-	  if(isalpha((++ptr)[0]))
+	  if(isalpha((unsigned char)((++ptr)[0])))
 	  {
 	    if((ptr[1] == ':')&&(ptr[2]=='/'))
 		{
@@ -439,7 +439,7 @@ GURL::GURL(const GUTF8String & url_in)
 GURL::GURL(const GNativeString & url_in)
   : url(url_in.getNative2UTF8()), validurl(false)
 {
-#if defined(WIN32) || defined(OS2)
+#if defined(_WIN32) || defined(OS2)
   if(is_valid() && is_local_file_url())
   {
     GURL::Filename::UTF8 xurl(UTF8Filename());
@@ -1071,7 +1071,7 @@ GURL::encode_reserved(const GUTF8String &gs)
   for (; *s; s++,d++)
   {
     // Convert directory separator to slashes
-#if defined(WIN32) || defined(OS2)
+#if defined(_WIN32) || defined(OS2)
     if (*s == backslash || *s== slash)
 #else
 #ifdef macintosh
@@ -1245,7 +1245,7 @@ GURL::GURL(const GNativeString &xurl,const GURL &codebase)
   GURL retval(xurl.getNative2UTF8(),codebase);
   if(retval.is_valid())
   {
-#if defined(WIN32)
+#if defined(_WIN32)
     // Hack for IE to change \\ to /
     if(retval.is_local_file_url())
     {
@@ -1321,16 +1321,16 @@ GURL::UTF8Filename(void) const
     else if ( !GStringRep::cmp(localhostspec2, url_ptr, sizeof(localhostspec2)-1 ) )
       // RFC 1738 local host form
       url_ptr += sizeof(localhostspec2)-1;
-    else if ( (strlen(url_ptr) > 4)   // "file://<letter>:/<path>"
-        && (url_ptr[0] == slash)      // "file://<letter>|/<path>"
-        && (url_ptr[1] == slash)
-        && isalpha(url_ptr[2])
-        && ( url_ptr[3] == colon || url_ptr[3] == '|' )
-        && (url_ptr[4] == slash) )
+    else if ( (strlen(url_ptr) > 4)         // "file://<letter>:/<path>"
+              && (url_ptr[0] == slash)      // "file://<letter>|/<path>"
+              && (url_ptr[1] == slash)
+              && isalpha((unsigned char)(url_ptr[2]))
+              && ( url_ptr[3] == colon || url_ptr[3] == '|' )
+              && (url_ptr[4] == slash) )
       url_ptr += 2;
     else if ( (strlen(url_ptr)) > 2 // "file:/<path>"
-        && (url_ptr[0] == slash)
-        && (url_ptr[1] != slash) )
+              && (url_ptr[0] == slash)
+              && (url_ptr[1] != slash) )
       url_ptr++;
 #endif
 
@@ -1352,7 +1352,7 @@ GURL::UTF8Filename(void) const
     retval = expand_name(url_ptr,root);
 #endif
     
-#if defined(WIN32) || defined(OS2)
+#if defined(_WIN32) || defined(OS2)
     if (url_ptr[0] && url_ptr[1]=='|' && url_ptr[2]== slash)
     {
       if ((url_ptr[0]>='a' && url_ptr[0]<='z') 
@@ -1397,7 +1397,7 @@ GURL::is_file(void) const
     {
       retval=!(buf.st_mode & S_IFDIR);
     }
-#elif defined(WIN32)
+#elif defined(_WIN32)
     GUTF8String filename(UTF8Filename());
     if(filename.length() >= MAX_PATH)
       {
@@ -1469,7 +1469,7 @@ GURL::is_dir(void) const
     {
       retval=(buf.st_mode & S_IFDIR);
     }
-#elif defined(WIN32)   // (either Windows or WCE)
+#elif defined(_WIN32)   // (either Windows or WCE)
     GUTF8String filename(UTF8Filename());
     if(filename.length() >= MAX_PATH)
       {
@@ -1533,7 +1533,7 @@ GURL::mkdir() const
         retval = 0;
       else 
         retval = ::mkdir(NativeFilename(), 0755);
-#elif defined(WIN32)
+#elif defined(_WIN32)
       if (is_dir())
         retval = 0;
       else 
@@ -1559,7 +1559,7 @@ GURL::deletefile(void) const
         retval = ::rmdir(NativeFilename());
       else
         retval = ::unlink(NativeFilename());
-#elif defined(WIN32)
+#elif defined(_WIN32)
       if (is_dir())
         retval = ::RemoveDirectoryA(NativeFilename());
       else
@@ -1589,7 +1589,7 @@ GURL::listdir(void) const
       retval.append(GURL::Native(de->d_name,*this));
     }
     closedir(dir);
-#elif defined (WIN32)
+#elif defined(_WIN32)
     GURL::UTF8 wildcard("*.*",*this);
     WIN32_FIND_DATA finddata;
     HANDLE handle = FindFirstFile(wildcard.NativeFilename(), &finddata);//MBCS cvt
@@ -1755,7 +1755,7 @@ GURL::expand_name(const GUTF8String &xfname, const char *from)
       EMPTY_LOOP;
     *s = 0;
   }
-#elif defined (WIN32) // WIN32 implementation
+#elif defined(_WIN32) // WIN32 implementation
   // Handle base
   strcpy(string_buffer, (char const *)(from ? expand_name(from) : GOS::cwd()));
   //  GNativeString native;
