@@ -27,8 +27,8 @@ extern "C" {
 #endif
 
 #ifndef MINILISPAPI
-# ifdef WIN32
-#  ifdef DLL_EXPORT
+# ifdef _WIN32
+#  ifdef MINILISPAPI_EXPORT
 #   define MINILISPAPI __declspec(dllexport)
 #  else
 #   define MINILISPAPI __declspec(dllimport)
@@ -39,6 +39,18 @@ extern "C" {
 # define MINILISPAPI /**/
 #endif
 
+#ifndef __cplusplus
+# ifndef inline
+#  if defined(__GNUC__)
+#   define inline __inline__
+#  elif defined(_MSC_VER)
+#   define inline __inline
+#  else
+#   define inline /**/
+#  endif
+# endif
+#endif
+
 #include <stddef.h>  
 
 /* -------------------------------------------------- */
@@ -63,13 +75,13 @@ typedef struct miniexp_s* miniexp_t;
    numbers, symbols, pairs, and objects.
    The latter category can represent any c++ object
    that inherits class <miniobj_t> defined later in this file.
-   The only such objects defined in this file are strings. */
+   Strings and floating point numbers are implemented this way.*/
 
 
 /* -------- NUMBERS -------- */
 
-/* Minilisp numbers can represent any integer 
-   in range [-2^29...2^29-1] */
+/* Minilisp numbers represent integers 
+   covering at least range [-2^29...2^29-1] */
 
 
 /* miniexp_numberp --
@@ -275,6 +287,40 @@ MINILISPAPI miniexp_t miniexp_concat(miniexp_t l);
 
 
 
+/* -------- OBJECTS (FLOATNUM) -------- */
+
+/* miniexp_floatnump --
+   Tests if an expression is an object
+   representing a floating point number. */
+
+MINILISPAPI int miniexp_floatnump(miniexp_t p);
+
+/* miniexp_floatnum --
+   Returns a new floating point number object. */
+
+MINILISPAPI miniexp_t miniexp_floatnum(double x);
+
+/* miniexp_doublep --
+   Tests if an expression can be converted
+   to a double precision number. */
+
+static inline int miniexp_doublep(miniexp_t p) {
+  return miniexp_numberp(p) || miniexp_floatnump(p);
+}
+
+/* miniexp_to_double --
+   Returns a double precision number corresponding to 
+   a lisp expression (a number or a floatnum.) */
+
+MINILISPAPI double miniexp_to_double(miniexp_t p);
+
+/* miniexp_double --
+   Returns a lisp expression representing a double
+   precision number. This will be a number if it fits
+   and a floatnum otherwise.
+ */
+
+MINILISPAPI miniexp_t miniexp_double(double x);
 
 
 /* -------------------------------------------------- */
@@ -324,7 +370,7 @@ MINILISPAPI miniexp_t miniexp_concat(miniexp_t l);
 
    * Only the following miniexp functions can cause a
      garbage collection: miniexp_cons(), miniexp_object(),
-     miniexp_string(), miniexp_substring(),
+     miniexp_string(), miniexp_substring(), miniexp_pname(),
      miniexp_concat(), miniexp_pprin(), miniexp_pprint(),
      miniexp_gc(), and minilisp_release_gc_lock().  A
      function that does not cause calls to these functions
@@ -336,13 +382,10 @@ MINILISPAPI miniexp_t miniexp_concat(miniexp_t l);
      everywhere.
 
    * Function arguments should remain <miniexp_t> in order
-     to allow interoperability with the C language. As a
-     consequence, functions must often copy their arguments
-     into minivars in order to make sure they remain
-     allocated. A small performance improvement can be
-     achieved by deciding that the function should always be
-     called using properly secured arguments. This is more
-     difficult to get right.
+     to allow interoperability with the C language. 
+     It is assumed that these arguments have been properly
+     secured by the caller and cannot disappear if a 
+     garbage collection occurs.
 
    C programs cannot use minivars as easily as C++ programs.
    Wrappers are provided to allocate minivars and to access
@@ -436,24 +479,38 @@ MINILISPAPI void minilisp_finish(void);
      * the parenthesis <(> and <)>,
      * the double quote <">,
      * the vertical bar <|>,
-     * the dieze character <#>, when followed by an 
-       ascii character with a non zero entry in the
-       dieze character array.
      * any other ascii character with a non zero entry 
        in the macro character array.
+     * the dieze character <#>, when followed by another
+       dieze or by an ascii character with a non zero entry 
+       in the dieze character array.
 
    - Symbols are represented by their name.
-     Vertical bars <|> can be used to delimit names that
-     contain blanks, special characters, non printable
-     characters, non ascii characters, or 
-     can be confused for a number.
+     Symbols whose name contains blanks, special characters, 
+     non printable characters, non ascii characters, 
+     or can be confused for a number are delimited
+     by vertical bars <|> and can contain two consecutive
+     vertical bars to represent a single vertical bar character.
      
    - Numbers follow the syntax specified by the C
-     function strtol() with base=0.
+     function strtol() with base=0, but are required
+     to start with a digit or with a sign character
+     followed by another character.
 
-   - Strings are delimited by double quotes.
-     All C string escapes are recognized.
-     Non printable ascii characters must be escaped.
+   - Floating point follow the syntax specified by the C
+     function strtod() with base=0, but are required
+     to start with a digit or with a sign character
+     followed by another character.
+
+   - Strings are delimited by double quotes. 
+     All non printable ASCII characters must be escaped. 
+     Besides all the usual C string escape sequences,
+     UTF8-encoded Unicode characters in range 0..0x10ffff
+     can be represented by escape sequence <\u> followed
+     by four hexadecimal digits or escape sequence <\U>
+     followed by six hexadecimal digits. Surrogate pairs
+     are always recognized as a single Unicode character.
+     The effect of invalid escape sequences is unspecified.
 
    - List are represented by an open parenthesis <(>
      followed by the space separated list elements,
@@ -489,18 +546,26 @@ MINILISPAPI miniexp_t miniexp_pname(miniexp_t p, int width);
    pointers <fputs>, <fgetc>, and <ungetc>, which are similar to their 
    stdio counterparts. Variable <data> defines four pointers that can 
    be used as a closure by the I/O functions.
-      When <p_print7bits> is nonzero and points to a nonzero integer, all
-   non-ascii characters in strings are output as octal escapes. When both
-   <p_macrochar> and <p_macroqueue> are non zero, a non zero entry in
-   <p_macrochar[c]> defines a special parsing function that is called when
-   <miniexp_read_r> encounters the character <c> (in range 0 to 127.) 
-   When both <p_diezechar> and <p_macroqueue> are non zero, a non zero 
-   entry in <p_diezechar[c]> defines a special parsing function that 
-   is called when <miniexp_read_r> encounters the character '#' followed
-   by character <c> (in range 0 to 127.)   These parsing functions return 
-   a list of <miniexp_t> that function <miniexp_read_r> returns one-by-one 
-   before processing more input. This list is in fact stored in the 
-   variable pointed by <io.p_macroqueue>.  */
+      Variable <p_flags> optionally points to a flag word that customize the
+   printing operation. All ASCII control characters present in strings are
+   displayed using C escapes sequences. Flag <miniexp_io_print7bits> causes
+   all other non ASCII characters to be escaped. Flag <miniexp_io_u6escape>
+   and <miniexp_io_u4escape> respectively authorize using the long and
+   short utf8 escape sequences "\U" and "\u". Their absence may force
+   using surrogate short escape sequences or only octal sequences.
+   Flag <miniexp_io_quotemoresyms> causes the output code to also quote
+   all symbols that start with a digit or with a sign character followed
+   by another character.
+      When both <p_macrochar> and <p_macroqueue> are non zero, a non zero 
+   entry in <p_macrochar[c]> defines a special parsing function that is called
+   when <miniexp_read_r> encounters the character <c> (in range 0 to 127.)
+   When both <p_diezechar> and <p_macroqueue> are non zero, a non zero entry
+   in <p_diezechar[c]> defines a special parsing function that is called when
+   <miniexp_read_r> encounters the character '#' followed by character <c> (in
+   range 0 to 127.)  These parsing functions return a list of <miniexp_t> that
+   function <miniexp_read_r> returns one-by-one before processing more
+   input. This list is stored in the variable pointed by <io.p_macroqueue>.  
+*/
 
 typedef struct miniexp_io_s miniexp_io_t;
 typedef miniexp_t (*miniexp_macrochar_t)(miniexp_io_t*);
@@ -511,19 +576,24 @@ struct miniexp_io_s
   int (*fgetc)(miniexp_io_t*);
   int (*ungetc)(miniexp_io_t*, int);
   void *data[4];
-  int *p_print7bits;
+  int *p_flags; /* previously named p_print7bits */
   miniexp_macrochar_t *p_macrochar;
   miniexp_macrochar_t *p_diezechar;
   minivar_t *p_macroqueue;
   minivar_t *p_reserved;
 };
 
+#define miniexp_io_print7bits          0x1
+#define miniexp_io_u4escape            0x2
+#define miniexp_io_u6escape            0x4
+#define miniexp_io_quotemoresymbols    0x20
+
 /* miniexp_io_init --
    Initialize a default <miniexp_io_t> structure
    that reads from stdin and prints to stdout. 
    Field <data[0]> is used to hold the stdin file pointer.
    Field <data[1]> is used to hold the stdout file pointer.
-   Fields <p_print7bits>, <p_macrochar>, <p_diezechar>
+   Fields <p_flags>, <p_macrochar>, <p_diezechar>
    and <p_macroqueue> are set to point to zero-initialized
    shared variables. */
 
@@ -573,25 +643,15 @@ MINILISPAPI miniexp_t miniexp_print(miniexp_t p);
 MINILISPAPI miniexp_t miniexp_pprin(miniexp_t p, int width);
 MINILISPAPI miniexp_t miniexp_pprint(miniexp_t p, int width);
 
-/* miniexp_macrochar, miniexp_macroqueue --
-   Function <miniexp_io_init> causes the corresponding pointers in the
-   <miniexp_io_t> structure to point to these variables.  A non zero entry in
-   <io.macrochar> defines a special parsing function that runs when
-   <miniexp_read_r> encounters the corresponding character. The parsing
-   function return a list of <miniexp_t> that function <miniexp_read_r>
-   returns one-by-one before processing more input. This list is in fact
-   stored in the variable pointed to by <io.macroqueue>. */
- 
-extern MINILISPAPI miniexp_macrochar_t miniexp_macrochar[128];
-extern MINILISPAPI minivar_t miniexp_macroqueue;
 
-
-/* Backward compatibility. */
+/* Backward compatibility (will eventually disappear) */
 extern MINILISPAPI int (*minilisp_puts)(const char *);
 extern MINILISPAPI int (*minilisp_getc)(void);
 extern MINILISPAPI int (*minilisp_ungetc)(int);
 extern MINILISPAPI miniexp_t (*minilisp_macrochar_parser[128])(void);
 extern MINILISPAPI miniexp_t (*minilisp_diezechar_parser[128])(void);
+extern MINILISPAPI miniexp_macrochar_t miniexp_macrochar[128];
+extern MINILISPAPI minivar_t miniexp_macroqueue;
 extern MINILISPAPI int minilisp_print_7bits;
 #if defined(stdin)
 MINILISPAPI void minilisp_set_output(FILE *f);
@@ -629,7 +689,7 @@ public:
   miniexp_t* operator&() { return &data; }
   minivar_t& operator=(miniexp_t p) { data = p; return *this; }
   minivar_t& operator=(const minivar_t &v) { data = v.data; return *this; }
-  ~minivar_t() { if ((*pprev = next)) next->pprev = pprev; }
+  ~minivar_t();
 #ifdef MINIEXP_IMPLEMENTATION
   static minivar_t *vars;
   static void mark(minilisp_mark_t*);
@@ -661,9 +721,9 @@ miniobj_t {
   /* pname: returns a printable name for this object.
      The caller must deallocate the result with delete[]. */
   virtual char *pname() const;
-  /* mark: iterates over miniexps contained by this object
+  /* mark: calls action() on all member miniexps of the object,
      for garbage collecting purposes. */
-  virtual void mark(minilisp_mark_t*);
+  virtual void mark(minilisp_mark_t *action);
   /* destroy: called by the garbage collector to
      deallocate the object. Defaults to 'delete this'. */
   virtual void destroy();
@@ -706,6 +766,15 @@ static inline miniobj_t *miniexp_to_obj(miniexp_t p) {
 MINILISPAPI miniexp_t miniexp_object(miniobj_t *obj);
 
 
+/* miniexp_mutate -- 
+   Atomically modifies a member of a garbage collected object. 
+   The object implementation must call this function to change 
+   the contents of a member variable <v> of object <obj>.
+   Returns <p>*/
+
+MINILISPAPI miniexp_t miniexp_mutate(miniexp_t obj, miniexp_t *v, miniexp_t p);
+
+
 #endif /* __cplusplus */
 
 
